@@ -9,7 +9,7 @@ import re
 import subprocess
 import time
 from pathlib import Path
-from typing import Any
+from typing import Any, Optional
 
 from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError
 
@@ -368,14 +368,16 @@ class Windows(OperatingSystem):
         windowtitle: str = None,
         wildcard: bool = False,
         timeout: int = 10,
-    ) -> bool:
+    ) -> Optional[int]:
         """Open associated application when opening file
 
+        Keyword `Open Dialog` is used if `windowtitle` is given.
+
         :param filename: path to file
         :param windowtitle: name of the window
         :param wildcard: set True for inclusive window title search, default False
         :param timeout: time to wait for dialog to appear
-        :return: True if application is opened, False if not
+        :return: application id or None
 
         Example:
 
@@ -392,7 +394,8 @@ class Windows(OperatingSystem):
             subprocess.call(["open", filename])
         else:
             subprocess.call(["xdg-open", filename])
-        returnvalue = True
+
+        app_instance = None
         if windowtitle:
             app_instance = self.open_dialog(
                 windowtitle, wildcard=wildcard, timeout=timeout
@@ -400,9 +403,8 @@ class Windows(OperatingSystem):
             if app_instance > 0:
                 self._apps[app_instance]["executable"] = filename
                 self._apps[app_instance]["startkeyword"] = "Open File"
-            else:
-                returnvalue = False
-        return returnvalue
+
+        return app_instance
 
     def open_executable(
         self,
@@ -853,9 +855,8 @@ class Windows(OperatingSystem):
         elements, _ = self.find_element(locator)
         if elements and len(elements) == 1:
             ctrl = elements[0]["control"]
-            if empty_field:
-                ctrl.type_keys("{VK_LBUTTON down}{VK_CLEAR}{VK_LBUTTON up}")
-            ctrl.type_keys(keys)
+            empty_method = "^a{BACKSPACE}" if empty_field else ""
+            ctrl.type_keys(f"{empty_method}{keys}")
         else:
             raise ValueError(f"Could not find unique element for '{locator}'")
 
@@ -1942,6 +1943,7 @@ class Windows(OperatingSystem):
                         "is_active": w.is_active(),
                         "keyboard_focus": w.has_keyboard_focus(),
                         "rectangle": [left, top, right, bottom],
+                        "object": w,
                     }
                 )
             except Exception as e:  # pylint: disable=broad-except
